# copyright (c) 2012/13 by Samuel de Ancos
# http://deancos.com | sdeancos@gmail.com

try:
    import urllib.request as urllib2
except:
    import urllib2
import json


class ClientBase (object):
    api_url = "https://api.carriots.com/"

    def __init__(self, api_key=None, client_type='json', protocol='v2'):
        self.api_key = api_key
        self.client_type = client_type
        self.protocol = protocol
        self.content_type = "application/vnd.carriots.api.v2+%s" % self.client_type
        self.headers = {'User-Agent': 'clicarriots',
                        'Content-Type': self.content_type,
                        'Accept': self.content_type,
                        'Carriots.apikey': self.api_key}
    
    def request(self, url, data='', method='GET'):
        if 'GET' == method:
            response = self.method_get(url)
        elif 'POST' == method:
            response = self.method_post(url, data)
        elif 'PUT' == method:
            response = self.method_put(url, data)
        elif 'DELETE' == method:
            response = self.method_delete(url)
        else:
            raise ValueError('method not valid')
        
        if '20' not in str(response.code):    
            return response.code, response.reason
        
        response_json_str = response.read().decode("utf-8")
        return response.code, json.loads(response_json_str)
    
    def method_get(self, url):
        request = urllib2.Request(url, headers=self.headers)
        
        try:
            response = urllib2.urlopen(request)
        except urllib2.HTTPError as e:
            return e

        return response
    
    def method_post(self, url, data):
        request = urllib2.Request(url, data=data, headers=self.headers)
        
        try:
            response = urllib2.urlopen(request)
        except urllib2.HTTPError as e:
            return e

        return response
    
    def method_put(self, url, data):
        opener = urllib2.build_opener(urllib2.HTTPHandler)
        request = urllib2.Request(url, data=data, headers=self.headers)

        request.get_method = lambda: 'PUT'
        
        try:
            response = opener.open(request)
        except urllib2.HTTPError as e:
            return e

        return response
    
    def method_delete(self, url):
        opener = urllib2.build_opener(urllib2.HTTPHandler)
        request = urllib2.Request(url, headers=self.headers)
            
        request.get_method = lambda: 'DELETE'
        
        try:
            response = opener.open(request)
        except urllib2.HTTPError as e:
            return e

        return response



# copyright (c) 2012/13 by Samuel de Ancos
# http://deancos.com | sdeancos@gmail.com

from json import dumps
from time import mktime
from datetime import datetime
from clicarriots.client import ClientBase


class Utils (ClientBase):
    
    def get_time_zones(self):
        url = ClientBase.api_url + "time_zones/"

        code, response = self.request(url)

        response = [time_zones['tz'] for time_zones in response['result']]
        return code, response
    
    def get_locales(self):
        url = ClientBase.api_url + "locales/"

        code, response = self.request(url)

        response = [locales['locale'] for locales in response['result']]
        return code, response


class Stream (ClientBase):
    
    def get(self, stream):
        url = ClientBase.api_url + "streams/"
        if stream:
            url = url + stream + "/"
        else:
            raise ValueError('stream value not valid')

        response = self.request(url)
        return response

    def send(self, device, data, at, type='stream'):
        if not isinstance(device, str):
            raise TypeError('device type not valid')
        
        if not isinstance(data, dict):
            raise TypeError('data type not valid')
        
        if not isinstance(at, datetime) and not 'now':
            raise TypeError('at type not valid')
        
        if not isinstance(type, str):
            raise TypeError('type type not valid')

        if type == 'stream':
            url = ClientBase.api_url + 'streams' 
        elif type == 'status':
            url = ClientBase.api_url + 'status'
        else:
            raise ValueError('type value not valid')
        
        if at != 'now':     
            at = int(mktime(at.timetuple()))

        stream = {"protocol": self.protocol,
                  "device": device,
                  "at": at,
                  "data": data}

        self.data = dumps(stream)

        response = self.request(url, data=self.data, method="POST")
        return response
    
    def list(self, params=None):
        url = ClientBase.api_url + "streams/?"
        url = get_params(url, params)
                
        response = self.request(url)
        return response
    
    def delete(self, stream):
        url = ClientBase.api_url + "streams/"
        if stream:
            url = url + stream + "/"
        else:
            raise ValueError('stream value not valid')

        response = self.request(url, method="DELETE")
        return response

class Device (ClientBase):
    
    def new(self, data):
        # Este metodo crea un Objecto device para create y update valido.
        pass

    def get(self, device):
        url = ClientBase.api_url + "devices/"
        if device:
            url = url + device + "/"
        else:
            raise ValueError('device value not valid')

        response = self.request(url)
        return response
    
    def list(self, params = None):
        url = ClientBase.api_url + "devices/?"
        url = get_params(url, params)
                
        response = self.request(url)
        return response
    
    def create(self, data):
        url = ClientBase.api_url + "devices/"
        
        # Falta validar DATA
                
        response = self.request(url, data=data, method="POST")
        return response
    
    def update(self, device, data):
        url = ClientBase.api_url + "devices/"
        if device:
            url = url + device + "/"
        else:
            raise ValueError('device value not valid')
        
        # Falta validar DATA
                
        response = self.request(url, data=data, method="PUT")
        return response
    
    def delete(self, device):
        url = ClientBase.api_url + "devices/"
        if device:
            url = url + device + "/"
        else:
            raise ValueError('device value not valid')

        response = self.request(url, method="DELETE")
        return response
    
    def get_types(self):
        url = ClientBase.api_url + "types/"

        code, response = self.request(url)

        response = [type['type'] for type in response['result']]
        return code, response
    
    def get_sensors(self):
        url = ClientBase.api_url + "sensors/"

        code, response = self.request(url)

        response = [sensor['sensor'] for sensor in response['result']]
        return code, response


class DeviceConfig (ClientBase):
    def create(self, device, data):
        url = ClientBase.api_url + "devices/"
        if device:
            url = url + device + "/deviceconfigs/" 
        else:
            raise ValueError('device value not valid')
                   
        response = self.request(url, data=data, method="POST")
        return response


class Dropbox (ClientBase):

    def get(self, username, op='', name='', mime=''):
        url = ClientBase.api_url + "external/dropbox/user/"
        if username:
            url = url + username
        else:
            raise ValueError('username value not valid')
        
        if op:
            url = url + "/?op=" + op
        else:
            raise ValueError('op value not valid')
        
        if op == 'file': 
            if name:
                url = url + "&name=" + name
            else:
                raise ValueError('name value not valid')
        
        if mime:
            url = url + "&mime=" + mime

        response = self.request(url)
        return response


def get_params(url, params):
    if params:
        if not isinstance(params, dict):
            raise TypeError('data type not valid')
        
        for key, value in params.items():
            url = url + key + "=" + value + "&"
        url = url[:-1]
    
    return url



from distutils.core import setup
setup(name='clicarriots',
      version='0.1',
      description = 'The unofficial simple client for carriots platform (in progress) (python 2.x/3.x)',
      author='Samuel de Ancos',
      author_email='sdeancos@gmail.com',
      url = 'https://github.com/sdeancos/clicarriots',
      download_url = 'https://github.com/sdeancos/clicarriots/tarball/0.1',
      packages=['clicarriots'],
      )
